@@ -9,7 +9,7 @@ events happen. The Domain provides access to Peers, Databases and Processes.
 from session import BaseListener, Session, SessionMonitor, SessionException
 from util import DatabaseAction, startProcess, killProcess, doDatabaseAction, queryEngine
 
-import time
+import time, json
 from threading import Event, Lock
 import xml.etree.ElementTree as ElementTree
 
@@ -91,6 +91,11 @@ class Domain(BaseListener):
     def disconnect(self):
         """Disconnect from the domain."""
         self.__monitor.close()
+        
+    def _send_domain_message(self, service, attributes=None, text=None, children=None):
+        session = Session(self.__entry_peer.address, port=self.__entry_peer.port, service=service)
+        session.authorize(self.__user, self.__password)
+        return session.doRequest(attributes, text, children)
 
     @property
     def user(self):
@@ -130,6 +135,61 @@ class Domain(BaseListener):
         """Return a list of databases in the domain"""
         return self.__databases.values()
 
+    def create_template(self, template_name, summary, requirements):
+        """Create template by name"""
+        response = self._send_domain_message(**Template.build_create_request(template_name, summary, requirements))
+        return ElementTree.fromstring(response).tag == Template.success_message
+
+    def update_template(self, template_name, summary, requirements):
+        """Update template by name"""
+        response = self._send_domain_message(**Template.build_update_request(template_name, summary, requirements))
+        return ElementTree.fromstring(response).tag == Template.success_message
+
+    def delete_template(self, template_name):
+        """Delete template by name"""
+        response = self._send_domain_message(**Template.build_delete_request(template_name))
+        return ElementTree.fromstring(response).tag == Template.success_message
+
+    def get_template(self, template_name):
+        """Return a template by name"""
+        response = self._send_domain_message(**Template.build_get_request(template_name))
+        return Template.from_message(response)
+
+    @property
+    def templates(self):
+        """Return a list of templates in the domain"""
+        response = self._send_domain_message(**Template.build_list_request())
+        return Template.from_list_message(response)
+
+    def create_description(self, name, template_name, variables, dba_user, dba_password):
+        response = self._send_domain_message(**Description.build_create_request(name, template_name, variables, dba_user, dba_password))
+        return ElementTree.fromstring(response).tag == Description.success_message
+
+    def update_description(self, name, template_name, variables):
+        response = self._send_domain_message(**Description.build_update_request(name, template_name, variables))
+        return ElementTree.fromstring(response).tag == Description.success_message
+
+    def delete_description(self, name):
+        response = self._send_domain_message(**Description.build_delete_request(name))
+        return ElementTree.fromstring(response).tag == Description.success_message
+
+    def get_description(self, name):
+        response = self._send_domain_message(**Description.build_get_request(name))
+        return Description.from_message(response)
+
+    def start_description(self, name):
+        response = self._send_domain_message(**Description.build_start_request(name))
+        return ElementTree.fromstring(response).tag == Description.success_message
+
+    def stop_description(self, name):
+        response = self._send_domain_message(**Description.build_stop_request(name))
+        return ElementTree.fromstring(response).tag == Description.success_message
+
+    @property
+    def descriptions(self):
+        response = self._send_domain_message(**Description.build_list_request())
+        return Description.from_list_message(response)
+
     def shutdown(self, graceful=True):
         """Shutdown all databases in the domain.
         
@@ -392,6 +452,31 @@ class Peer:
         """Return True if this peer is a broker."""
         return self.__is_broker
 
+    @property
+    def tags(self):
+        """Return all host tags"""
+        message = self.__domain._send_domain_message("Tag", {'Action': 'GetHostTags', 'AgentId': self.id})
+        tags = ElementTree.fromstring(message)
+        data = {}
+        for tag in tags:
+            data[tag.get('Key')] = tag.get('Value')
+
+        return data        
+    
+    def get_tag(self, tag):
+        """Return host tag"""
+        return self.tags[tag]
+
+    def set_tag(self, key, value):
+        """Set host tag"""
+        element = ElementTree.fromstring("<Tag Key=\"%s\" Value=\"%s\"/>" % (key, value))
+        self.__domain._send_domain_message("Tag", {'Action': 'SetHostTags', 'AgentId': self.id}, children=[element])
+        
+    def delete_tag(self, key):
+        """Delete host tag"""
+        element = ElementTree.fromstring("<Tag Key=\"%s\"/>" % (key))
+        self.__domain._send_domain_message("Tag", {'Action': 'DeleteHostTags', 'AgentId': self.id}, children=[element])
+
     def start_transaction_engine(self, db_name, options=None, wait_seconds=None):
         """Start a transaction engine on this peer for a given database.
         
@@ -556,6 +641,37 @@ class Database:
         """Return the name of this database."""
         return self.__name
 
+    @property
+    def description(self):
+        """Return the description of this database."""
+        message = self.__domain._send_domain_message("Description", {'Action': 'GetDatabaseDescription', 'DatabaseName': self.__name})
+        return json.loads(ElementTree.fromstring(message).text)
+
+    @property
+    def status(self):
+        """Return the status of the database."""
+        #TODO: hack to determine database state
+        data = {'RUNNING': 0, 'QUIESCED': 0}
+        for process in self.processes:
+            if process.status == "RUNNING":
+                data['RUNNING'] = data['RUNNING'] + 1
+            if process.status == "QUIESCED":
+                data['QUIESCED'] = data['QUIESCED'] + 1
+        if data['RUNNING'] > data['QUIESCED']:
+            return "RUNNING"
+        if data['QUIESCED'] > data['RUNNING']:
+            return "QUIESCED"
+
+    @property
+    def storage_managers(self):
+        """Return storage managers."""
+        return [process for process in self.__processes.values() if not process.is_transactional]
+      
+    @property
+    def transaction_engines(self):
+        """Return transaction engines."""
+        return [process for process in self.__processes.values() if process.is_transactional]
+
     def _add_process(self, process):
         self.__processes[self.__process_id(process)] = process
 
@@ -839,3 +955,193 @@ class Process:
 
         return queryEngine(self.address, self.port, query_type, pwd, msg_body)
     
+class Template:
+    success_message = "Success"
+    @staticmethod
+    def build_create_request(name, summary, requirements):
+        summary_element = ElementTree.Element("Summary")
+        summary_element.text = summary
+        requirements_element = ElementTree.Element("Requirements")
+        requirements_element.text = requirements
+        return {"service": "Description",
+                "attributes": {'Action': 'CreateTemplate', 'TemplateName': name},
+                "children": [summary_element, requirements_element]}
+
+    @staticmethod
+    def build_update_request(name, summary, requirements):
+        summary_element = ElementTree.Element("Summary")
+        summary_element.text = summary
+        requirements_element = ElementTree.Element("Requirements")
+        requirements_element.text = requirements
+        return {"service": "Description",
+                "attributes": {'Action': 'UpdateTemplate', 'TemplateName': name},
+                "children": [summary_element, requirements_element]}
+
+    @staticmethod
+    def build_delete_request(name):
+        return {"service": "Description", "attributes": {'Action': 'DeleteTemplate', 'TemplateName': name}}
+
+    @staticmethod
+    def build_get_request(name):
+        return {"service": "Description", "attributes": {'Action': 'GetTemplate', 'TemplateName': name}}
+
+    @staticmethod
+    def build_list_request():
+        return {"service": "Description", "attributes": {'Action': 'ListTemplates'}}
+
+    @staticmethod
+    def from_message(message):
+        root = ElementTree.fromstring(message)
+        name = root.get("TemplateName")
+
+        summary = ""
+        summary_element = root.find("Summary")
+        if summary_element is not None:
+            summary = summary_element.text
+
+        requirements = ""
+        requirements_element = root.find("Requirements")
+        if requirements_element is not None:
+            requirements = requirements_element.text
+
+        return Template(name, summary, requirements)
+
+    @staticmethod
+    def from_list_message(message):
+        names = list()
+        root = ElementTree.fromstring(message)
+        for child in root:
+            names.append(child.get("TemplateName"))
+
+        return names
+
+
+    def __init__(self, name, summary, requirements):
+        self._name = name
+        self._summary = summary
+        self._requirements = requirements
+
+    @property
+    def name(self):
+        return self._name
+
+    @property
+    def summary(self):
+        return self._summary
+
+    @property
+    def requirements(self):
+        return self._requirements
+
+
+
+
+class Description:
+    success_message = "Success"
+    @staticmethod
+    def build_create_request(name, template_name, variables, dba_user, dba_password):
+        template_element = ElementTree.Element("Template")
+        template_element.text = template_name
+        variables_element = ElementTree.Element("Variables")
+        for key in variables:
+            variable_child = ElementTree.SubElement(variables_element, "Variable")
+            variable_child.set("Key", key)
+            variable_child.text = variables[key]
+
+        return {"service": "Description",
+                "attributes": {'Action': 'CreateDescription',
+                               'DatabaseName': name,
+                               'DbaUser': dba_user,
+                               'DbaPassword': dba_password},
+                "children": [template_element, variables_element]}
+
+    @staticmethod
+    def build_update_request(name, template_name, variables):
+        template_element = ElementTree.Element("Template")
+        template_element.text = template_name
+        variables_element = ElementTree.Element("Variables")
+        for key in variables:
+            variable_child = ElementTree.SubElement(variables_element, "Variable")
+            variable_child.set("Key", key)
+            variable_child.text = variables[key]
+
+        return {"service": "Description",
+                "attributes": {'Action': 'UpdateDescription',
+                               'DatabaseName': name},
+                "children": [template_element, variables_element]}
+
+    @staticmethod
+    def build_delete_request(name):
+        return {"service": "Description", "attributes": {'Action': 'DeleteDescription', 'DatabaseName': name}}
+
+    @staticmethod
+    def build_get_request(name):
+        return {"service": "Description", "attributes": {'Action': 'GetDescription', 'DatabaseName': name}}
+
+    @staticmethod
+    def build_list_request():
+        return {"service": "Description", "attributes": {'Action': 'ListDescriptions'}}
+
+    @staticmethod
+    def build_start_request(name):
+        return {"service": "Description", "attributes": {'Action': 'StartDescription', 'DatabaseName': name}}
+
+    @staticmethod
+    def build_stop_request(name):
+        return {"service": "Description", "attributes": {'Action': 'StopDescription', 'DatabaseName': name}}
+
+    @staticmethod
+    def from_message(message):
+        root = ElementTree.fromstring(message)
+        name = root.get("DatabaseName")
+
+        template_name = ""
+        template_element = root.find("Template")
+        if template_element is not None:
+            template_name = template_element.text
+
+        variables = {}
+        variables_element = root.find("Variables")
+        if variables_element is not None:
+            for var in variables_element:
+                key = var.get("Key")
+                value = var.text
+                variables[key] = value
+
+        status = ""
+        status_element = root.find("Status")
+        if status_element is not None:
+            status = status_element.text
+
+        return Description(name, template_name, variables, status)
+
+    @staticmethod
+    def from_list_message(message):
+        names = list()
+        root = ElementTree.fromstring(message)
+        for child in root:
+            names.append(child.get("DatabaseName"))
+
+        return names
+
+    def __init__(self, name, template_name, variables, status):
+        self._name = name
+        self._template_name = template_name
+        self._variables = variables
+        self._status = status
+
+    @property
+    def name(self):
+        return self._name
+
+    @property
+    def template_name(self):
+        return self._template_name
+
+    @property
+    def variables(self):
+        return self._variables
+
+    @property
+    def status(self):
+        return self._status
\ No newline at end of file
